import unittest
from zope.interface.verify import verifyObject
from zope.app.testing import ztapi

from bibliograph.parsing.tests import setup
from bibliograph.parsing.tests.base import TestEntries
from bibliograph.parsing.parsers.endnote import EndNoteParser
from bibliograph.parsing.interfaces import IBibliographyParser

class TestEndNoteParser(unittest.TestCase):

    def setUp(self):
        self.parser = EndNoteParser()

        from bibliograph.rendering.interfaces import IBibTransformUtility
        from bibliograph.rendering.utility import ExternalTransformUtility    
        ztapi.provideUtility(IBibTransformUtility, ExternalTransformUtility(),
                             name=u'external')

    def test_parser_contract(self):
        self.failUnless(IBibliographyParser.providedBy(EndNoteParser()))
        self.failUnless(verifyObject(IBibliographyParser, EndNoteParser()))

    def test_import(self):

        if self.parser.isAvailable():
            source = open(setup.ENDNOTE_TEST_SOURCE, 'r').read()
            self.failUnless(source)
            entries = TestEntries(self.parser.getEntries(source))
            self.failUnless(len(entries) == 2)
            expected_author_last_names = ('Dufour', 'Fieschi', 'Hergon',
                                          'Joubert', 'Raps', 'Staccini',)

            parsed_author_last_names = entries.author_last_names()
            for name in expected_author_last_names:
                self.failUnless(name in parsed_author_last_names,
                                'Parse failed - missing author %s' % name)

            expected_titles = ('Combining advanced networked technology and pedagogical methods to improve collaborative distance learning',
                               'Collaborative and workflow-oriented digital portfolio: Creating a web-based tool to support a nationwide program of practices evaluation in the blood transfusion area',)
                               
            parsed_titles = [e.title for e in entries.entries]
            for title in expected_titles:
                self.failUnless(title in parsed_titles,
                                'Parse failed - missing expected title "%s"' %
                                title)
            # Keywords - first entry in sample file has none, second has 9
            self.failIf(hasattr(entries.entries[0],'keywords')
                        and entries.entries[0].keywords,
                        'Parsed non existing keywords')
            self.failUnless(len(entries.entries[1].keywords) == 9)
            # Confirm a few w/ tricky characters survived the parse
            kwds = ["Health Services Research/ organization \\& administration",
                    "Primary Health Care/ standards",
                    "Program Evaluation",
                    "Quality Assurance, Health Care"]
            for keyword in kwds:
                self.failUnless(keyword in entries.entries[1].keywords,
                                "Failed to correctly parse keyword: '%s'" % keyword)
        else:
            print """\nOne or more transformationtool was not found!
please make sure bibutils is installed to run all tests. """
            print ("-" * 20) + "\n"
        
    def test_check_format(self):
        s0 = open(setup.ENDNOTE_TEST_SOURCE, 'r').read()
        s1 = open(setup.RIS_SOURCE, 'r').read()
        
        self.failUnless(self.parser.checkFormat("%0 \n%A \n%D \n%T "))
        self.failIf(self.parser.checkFormat("%0 \n%A \n%T "))
        self.failUnless(self.parser.checkFormat(s0), 
                        'Endnote Parser failed to detect Endnote format')
        self.failIf(self.parser.checkFormat(s1), 
                        'Endnote Parser incorrectly accepted RIS format')

def test_suite():
    suite = unittest.TestSuite([
        unittest.makeSuite(TestEndNoteParser),])
    return suite



# -*- coding: utf-8 -*-
#######################################################
#                                                     #
#    Copyright (C), 2004, Raphael Ritz                #
#    <r.ritz@biologie.hu-berlin.de>                   #
#                                                     #
#    Humboldt University Berlin                       #
#                                                     #
# Copyright (C), 2005, Logilab S.A. (Paris, FRANCE)   #
# http://www.logilab.fr/ -- mailto:contact@logilab.fr #
#                                                     #
#######################################################
import unittest

from bibliograph.parsing.parsers.bibtex import BibtexParser
from bibliograph.parsing.tests import setup
from bibliograph.parsing.tests.base import TestEntries

class TestBibtexParsing(unittest.TestCase):
    """
    """

    def setUp(self):
        self.parser = BibtexParser()

    def testFormatDetection(self):
        source_files = (setup.MEDLINE_TEST_BIB, setup.BIBTEX_TEST_BIB, 
                        setup.IDCOOKING_TEST_BIB, setup.PDFFOLDER_TEST_BIB, 
                        setup.BIBTEX_TEST_BIB_DUP, setup.BIBTEX_TEST_MULTI_AUTHORS,
                        setup.BIBTEX_TEST_INBOOKREFERENCES, setup.BIBTEX_TEST_LASTFIELDKOMMA,
                        setup.BIBTEX_TEST_TYPEFIELD, setup.BIBTEX_TEST_CITE_KEY)

        for source_file in source_files:
            source = open(source_file, 'r').read()
            self.failUnless(self.parser.checkFormat(source), 'BibTeX parser failed to detect BibTeX format in file %s' % source_file)

        # check negative detection (check properly rejects non-bibtex format files)
        source = open(setup.MEDLINE_TEST_MED, 'r').read()
        self.failIf(self.parser.checkFormat(source), 'BibTeX parser incorrectly detected BibTeX format in file %s' % setup.MEDLINE_TEST_MED)

    def testBibtexAuthorParsing(self):
        source = open(setup.BIBTEX_TEST_MULTI_AUTHORS, 'r').read()
        source = self.parser.preprocess(source)
        result = self.parser.parseEntry(source)
        heckman =  {'middlename': 'J.',
                    'firstname' : 'James',
                    'lastname'  : 'Heckman'}
        carneiro = {'middlename': '',
                    'firstname' : 'Pedro',
                    'lastname'  : 'Carneiro'}
        self.failUnless( len( result['authors'] ) == 2 )
        author1 = result['authors'][0]
        self.failUnless(author1['middlename'] == carneiro['middlename'])
        self.failUnless(author1['firstname'] == carneiro['firstname'])
        self.failUnless(author1['lastname'] == carneiro['lastname'])
        author2 = result['authors'][1]
        self.failUnless(author2['middlename'] == heckman['middlename'])
        self.failUnless(author2['firstname'] == heckman['firstname'])
        self.failUnless(author2['lastname'] == heckman['lastname'])

    def testBibtexInbookReferenceParsing(self):
        source = open(setup.BIBTEX_TEST_INBOOKREFERENCES, 'r').read()
        ref = {
            'booktitle': 'In einem fiktiven Buch vor unserer Zeit',
            'title': 'Die Tage der Ankunft',
            'chapter': 'Die Tage der Ankunft',
            'publication_url': 'http://www.sunweavers.net/',
        }

        source = self.parser.preprocess(source)
        result = self.parser.parseEntry(source)

        for key in ref.keys():
            self.failUnless( result.has_key(key) and (ref[key] == result[key]),key )

    def testAnnoteParsing(self):
        source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        results = self.parser.getEntries(source)
        self.failUnless(results[-1]['annote'] == 'I really like it.')

    def testIdentifierParsing(self):
        source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        results = self.parser.getEntries(source)
        result = results[2]
        self.assertEqual(result['identifiers'], [{'label' : 'ISBN', 'value' : '3874402436'},
                                                 {'label' : 'DOI', 'value' : '1-23-345'}])

    def testBibtexTypeFieldParsing(self):
        source = open(setup.BIBTEX_TEST_TYPEFIELD, 'r').read()
        ref = {
            'publication_type': 'Doktorarbeit',
            'title': 'Mein Herr Doktor',
            'school': 'CAU Kiel',
            'institution': 'Ã–kologie-Zentrum',
        }

        source = self.parser.checkEncoding(source)
        source = self.parser.preprocess(source)
        result = self.parser.parseEntry(source)

        for key in ref.keys():
            self.failUnless( result.has_key(key) and (ref[key] == result[key]) )

    def testBibtexTypeLastFieldTrailingKomma(self):
        source = open(setup.BIBTEX_TEST_LASTFIELDKOMMA, 'r').read()
        results = self.parser.getEntries(source)

        # the last field in a bibtex entry always had a trailing ","
        self.failUnless( len(results) == 2  )
        self.failUnless( results[0]['institution'] == results[1]['institution']  )
        self.failUnless( results[0]['publication_type'] == results[1]['publication_type']  )
        self.failUnless( results[0]['publication_type'] == 'Doktorarbeit,,,' )

class TestBibtexParsing2(unittest.TestCase):
    """ more tests """

    def setUp(self):
        self.parser = BibtexParser()

    def testBibtexWithCustomFieldnames(self):
        source = open(setup.BIBTEX_TEST_BIB2, 'r').read()
        results = self.parser.getEntries(source)
        r1 = results[0]
        self.assertEqual(r1['month'], 'Mar')
        self.assertEqual(r1['doi'], '10.1002/(ISSN)1097-0231')
        self.assertEqual(r1['date-added'], '2008-08-06 17:48:48 +0200')
        self.assertEqual(r1['rating'], '0')
        self.assertEqual(r1['keywords'], ['biology', 'chemistry'])
        r2 = results[1]
        self.assertEqual(r2['keywords'], ['something strange'])

    def testBibtexEncodedChars(self):
        source = open(setup.BIBTEX_TEST_BIB3, 'r').read()
        results = self.parser.getEntries(source)
        r = results[0]
        self.assertEqual(r['title'], unicode('Der Fürst', 'iso-8859-15').encode('utf-8'))
        self.assertEqual(r['publisher'], unicode('Alfred Körner Verlag', 'iso-8859-15').encode('utf-8'))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBibtexParsing))
    suite.addTest(makeSuite(TestBibtexParsing2))
    return suite

if __name__ == '__main__':
    framework()



############################################################################
#                                                                          #
#             copyright (c) 2004 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""XMLParser (MODS) class"""

# Python stuff
import re, os

# Zope stuff
from zope.component import getUtility

# Bibliography stuff
from bibliograph.rendering.interfaces import IBibTransformUtility
from bibliograph.parsing.parsers.base import isTransformable
from bibliograph.parsing.parsers.bibtex import BibtexParser as BaseParser


class XMLParser(BaseParser):
    """
    A specific parser to process input in XML(MODS)-format. The XML intermediate format is conform to the
    Library of Congress's Metadata Object Description Schema (MODS). This is a very flexible standard that
    should prove quite useful as the number of tools that directly interact with it increase.
    """

    meta_type = "XML (MODS) Parser"

    format = {'name':'XML (MODS)',
              'extension':'xml'}

    def __init__(self,
                 id = 'xml_mods',
                 title = "XML(MODS) parser"
                 ):
        """
        initializes including the regular expression patterns
        """
        BaseParser.__init__(self, id=id, title=title)

    # Here we need to provide 'isAvailable', 'checkFormat' and 'preprocess'

    def isAvailable(self):
        """ test if transforming from XML to BibTex is possible...
        """
        return isTransformable('xml', 'bib')

    def checkFormat(self, source):
        """ is this my format?
        """
        teststring = source[:200].lower()
        ai = teststring.find('www.loc.gov/mods')
        ei = teststring.find('modsCollection')
        if ai + ei > -2:
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        convert XML to BibTeX
        """
        tool = getUtility(IBibTransformUtility, name=u"external")
        return tool.transform(source, 'xml', 'bib')

    # all the rest we inherit from our parent BibTeX(!) parser



from zope.interface import Interface


class IBibliographyParser(Interface):
    """ Interface for bibliographic input parsers
    """

    def isAvailable():
        """
        are all pre-requisites for this parser fullfilled?
        """

    def isEnabled():
        """
        is this parser enabled by the portal manager (default: yes)?
        """

    def getFormatName():
        """
        returns the name of the format
        """

    def getFormatExtension():
        """
        returns the filename extension of the format
        """

    def getDelimiter():
        """
        returns the delimiter used to split a list of entries into pieces
        """

    def getPattern():
        """
        returns the pattern used to parse a single entry
        """

    def setDelimiter(delimiter="\n\n", flags=None):
        """
        sets the delimiter used to split a list of entries into pieces
        the 'delimiter' argument and the flags are passed to 're.compile'
        """

    def setPattern(pattern="\n", flags=None):
        """
        sets the pattern used to parse a single entry
        the 'pattern' argument is passed to 're.compile')
        """

    def checkFormat(source):
        """
        checks whether source has the right format
        returns true (1) if so and false (0) otherwise
        """

    def getEntries(source):
        """
        splits a (text) file with several entries
        parses the entries
        returns a list of the parsed entries
        """

    def splitSource(source):
        """
        splits a (text) file with several entries
        returns a list of those entries
        """

    def parseEntry(entry):
        """
        parses a single entry

        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """



# This is so we can find our sample test files
from bibliograph.parsing.tests import GLOBALS
from Globals import package_home
PACKAGE_HOME = package_home(GLOBALS)

from os.path import join
MEDLINE_TEST_MED = join(PACKAGE_HOME, 'samples', 'medline_test.med')
MEDLINE_TEST_BIB = join(PACKAGE_HOME, 'samples', 'medline_test.bib')
MEDLINE_TEST_XML = join(PACKAGE_HOME, 'samples', 'medline_test.xml')
BIBTEX_TEST_BIB = join(PACKAGE_HOME, 'samples', 'bibtex_test.bib')
BIBTEX_TEST_BIB2 = join(PACKAGE_HOME, 'samples', 'bibtex_test2.bib')
BIBTEX_TEST_BIB3 = join(PACKAGE_HOME, 'samples', 'bibtex_test3.bib')
IDCOOKING_TEST_BIB = join(PACKAGE_HOME, 'samples', 'idcooking_test.bib')
PDFFOLDER_TEST_BIB = join(PACKAGE_HOME, 'samples', 'pdffolder_test.bib')
CMFBAT_TEST_PDF_1 = join(PACKAGE_HOME, 'samples', 'cmfbat-pdffile1.pdf')
CMFBAT_TEST_PDF_2 = join(PACKAGE_HOME, 'samples', 'cmfbat-pdffile2.pdf')
CMFBAT_TEST_PDF_3 = join(PACKAGE_HOME, 'samples', 'cmfbat-pdffile3.pdf')
BIBTEX_TEST_BIB_DUP = join(PACKAGE_HOME, 'samples', 
                           'bibtex_test_duplicates.bib')
BIBTEX_TEST_MULTI_AUTHORS = join(PACKAGE_HOME, 'samples',
                                 'bibtex_test_multiauthors.bib')
BIBTEX_TEST_INBOOKREFERENCES = join(PACKAGE_HOME, 'samples',
                                 'bibtex_test_inbookreferences.bib')
BIBTEX_TEST_LASTFIELDKOMMA = join(PACKAGE_HOME, 'samples',
                                 'bibtex_test_lastfieldkomma.bib')
BIBTEX_TEST_TYPEFIELD = join(PACKAGE_HOME, 'samples',
                                 'bibtex_test_typefield.bib')
BIBTEX_TEST_CITE_KEY = join(PACKAGE_HOME, 'samples',
                            'bibtex_test_cite_key.bib')
RIS_SOURCE = join(PACKAGE_HOME, 'samples', 'ris_test.ris')
ENDNOTE_TEST_SOURCE = join(PACKAGE_HOME, 'samples', 'endnote.end')



from zope.interface import implements
from bibliograph.core.interfaces import IBibliographicReference

class TestEntries(object):
    """ For testing the parsers, this class wraps breaking the parsed
    dictionaries into TestEntry objects to simplify the testing code.
    Typically, application code will build its own entry type. 
    """

    def __init__(self, parsed_entries):
        """ Given a list of parsed entries, builds a list
        of TestEntry objects @ self.entries
        """
        self.entries = []

        # Some parsers leave list items containing notes about the 
        # parsing process - if so, toss out the notes and just hang
        # onto the entries themselves
        for e in parsed_entries:
            if len(e.keys()) > 2:
                self.entries.append(TestEntry(e))

        #for i in range(0, len(parsed_entries)-1, 2):
        #    self.entries.append(TestEntry(parsed_entries[i]))

    def __len__(self):
        return len(self.entries)

    def author_last_names(self):
        """ Returns a list of all author last names contained in 
        the respective entries.
        """
        last_names = []
        for e in self.entries:
            for a in e.authors:
                last_names.append(a['lastname'])
        return last_names

    def titles(self):
        """ Returns a list of all titles from respective entries.
        """
        return [e.title for e in self.entries]
        
    def entryByTitle(self, title):
        """ Takes a title, returns the entry which has that title, or None
        """
        for e in self.entries:
            if e.title == title:
                return e
        return None

class TestEntry(object):
    """ Intended for test purposes only - takes a parsed entry and
    generates a more logical structure for testing contents."""

    def __init__(self, kw):
        '''Expects a single parsed entry from one of the parsers.  If all
        of the required fields aren't present, an exception is raised.
        '''

        # Required fields
        required = ['authors', 'publication_month', 'publication_year',
                    'reference_type', 'title', 'volume']
        for key in required:
            try:
                setattr(self,key,kw[key])
            except:
                raise Exception("Doesn't look like a valid parsed entry. " +
                                "Missing at least field: %s from %s" % (key,kw))

        # Optional fields
        for key in kw.keys():
            if key not in required:
                setattr(self,key,kw[key])

                
    def authorIsPresent(self, author_dict):
        """ Expects a dictionary with keys 'firstname', 'middlename', 'lastname'
            Returns true if there is an author in self.authors which matches
        """
        for a in self.authors:
            dict_same = True
            for key in a.iterkeys():
                if a[key] != author_dict[key]:
                    dict_same = False
            if dict_same == True:
                return True
            
        return False



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""BibliographyParser main class"""

# Python stuff
import re

# Zope stuff
from zope.interface import implements

from bibliograph.parsing.interfaces import IBibliographyParser
from bibliograph.core.bibutils import _getCommand


class BibliographyParser(object):
    """
    Base class for the input parser of the bibliography tool.
    """
    implements(IBibliographyParser)

    parser_enabled = True
    format = {'name':'name of the format',
              'extension':'typical filename extension'
              }

    delimiter = "\n\n"       # used to split a text into a list of entries
    pattern = r'(^.{0,4}- )' # the Medline pattern as default

    def __init__(self):
        """
        minimal initialization
        """
        self.format = BibliographyParser.format

    def isAvailable(self):
        """ by default parser is available, override in specific parser's code
            if there are some hurdles to take before parsing is possible
        """
        return True

    def isEnabled(self):
        """ if parser is enabled or not can be configured in the PropertyManager
        """
        return self.parser_enabled

    def Description(self):
        """
        a short text that explains the target format of the parser
        """
        # TODO i18n this method, port to bibliograph.parsing domain
        # domain='cmfbibliographyat'
        # msgid='help_parser_%s' % self.getId()
        # return self.translate(domain=domain, msgid=msgid, default='%s' % self.__doc__)
        return self.__doc__

    def getFormatName(self):
        """ returns the name of the format """
        return self.format.get('name', 'No name specified')

    def getFormatExtension(self):
        """ returns the filename extension of the format """
        return self.format.get('extension', 'no extension specified')

    def getDelimiter(self):
        """
        returns the delimiter used to split a list of entries into pieces
        """
        return self.delimiter

    def getPattern(self):
        """
        returns the pattern used to parse a single entry
        """
        return self.pattern

    def setDelimiter(self, delimiter="\n\n", flags=None):
        """
        sets the delimiter used to split a list of entries into pieces
        the 'delimiter' argument and the flags are passed to 're.compile'
        """
        if flags:
            self.delimiter = re.compile(delimiter, flags)
        else:
            self.delimiter = re.compile(delimiter)
        return None

    def setPattern(self, pattern="\n", flags=None):
        """
        sets the pattern used to parse a single entry
        the 'pattern' argument is passed to 're.compile')
        """
        if flags:
            self.pattern = re.compile(pattern, flags)
        else:
            self.pattern = re.compile(pattern)
        return None

    def checkFormat(self, source):
        """
        checks whether source has the right format
        returns true (1) if so and false (0) otherwise
        """
        pass # needs to be provided by the individual parsers

    def splitSource(self, source):
        """
        splits a (text) file with several entries
        returns a list of those entries
        """
        if hasattr(self, 'preprocess'):
            source = self.preprocess(source)
        return self.delimiter.split(source)

    def parseEntry(self, entry):
        """
        parses a single entry

        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        pass  # needs to be overwriten by the individual parser

    def getEntries(self, source):
        """
        splits a (text) file with several entries
        parses the entries
        returns a list of the parsed entries
        """
        source = self.checkEncoding(source)
        return [self.parseEntry(entry) \
                for entry in self.splitSource(source)]

    def checkEncoding(self, source):
        """
        Make sure we have utf encoded text
        """
        try:
            source = unicode(source, 'utf-8')
        except UnicodeDecodeError:
            source = unicode(source, 'latin1')
        return source.encode('utf-8')


class EntryParseError(object):
    """Parsers can return instances of this class when the parsing
    of an entry fails for whatever reason.
    The description is then taken and added to the import_report.
    """

    def __init__(self, description):
        self.description = description.replace('\n', '\\n')

    def get(self, attr_name):
        # XXX Hack! import.cpy does::
        #    if entry.get('title'):
        #        ...
        # So we need to mimic that here
        if attr_name == 'description':
            return self.description
        else:
            return 'EntryParseError'

def isTransformable(source_format, target_format):
    """
    test if a transform from source_format to target_format.
    would be feasible
    """
    try:
        _getCommand(source_format, target_format)
    except (ValueError, LookupError):
        return False

    return True



import unittest
from zope.interface.verify import verifyObject

from bibliograph.parsing.interfaces import IBibliographyParser
from bibliograph.parsing.parsers.base import BibliographyParser


class BibliographyParserTest(unittest.TestCase):
    '''Tests for the parser base class'''

    def test_init(self):
        self.failUnless(BibliographyParser())

    def test_provides(self):
        self.failUnless(IBibliographyParser.providedBy(BibliographyParser()))

    def test_verify(self):
        self.failUnless(verifyObject(IBibliographyParser, BibliographyParser()))

def test_suite():
    suite = unittest.TestSuite([
        unittest.makeSuite(BibliographyParserTest),])
    return suite



############################################################################
#                                                                          #
#             copyright (c) 2004, 2006 ITB, Humboldt-University Berlin     #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""EndNoteParser class"""
# Python stuff
import re

# Zope stuff
from zope.component import getUtility 
from zope.interface import implements

# Bibliography stuff
from bibliograph.rendering.interfaces import IBibTransformUtility
from bibliograph.parsing.parsers.base import isTransformable
from bibliograph.parsing.parsers.bibtex import BibtexParser as BaseParser


class EndNoteParser(BaseParser):
    """
    A specific parser to process input in EndNote's text format.
    """

    format = {'name':'EndNote',
              'extension':'end'}

    def __init__(self,
                 id = 'endnote',
                 title = "EndNote's text format parser"
                 ):
        """
        initializes including the regular expression patterns
        """
        BaseParser.__init__(self, id=id, title=title)

    # Here we need to provide 'checkFormat' and 'preprocess'

    def isAvailable(self):
        """ test if transforming from Endnote to BibTex is possible...
        """
        return isTransformable('end', 'bib')

    def checkFormat(self, source):
        """
        does the source look to be in endnote format?
        """
        # Ultimately end2xml is used
        # http://www.scripps.edu/~cdputnam/software/bibutils/bibutils2.html#end2xml

        pattern = re.compile('^%[0-9|A-Z] ', re.M)
        all_tags = re.findall(pattern, source)
        # Should always start w/ '%0' and have at least one author '%A',
        # a year (date) '%D' and a title '%T'
        required = ('%A ', '%D ', '%T ')
        if len(all_tags) and all_tags[0] == '%0 ' and \
                reduce(lambda i, j: i and j, [r in all_tags for r in required]):
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        prep the source and convert EndNote to BibTeX
        """
        # Remove carriage returns - end2xml chokes on them
        source = source.replace('\r', '')

        tool = getUtility(IBibTransformUtility, name=u"external")
        return tool.transform(source, 'end', 'bib')

    # all the rest we inherit from our parent BibTeX(!) parser



import unittest
from zope.interface.verify import verifyObject

from bibliograph.parsing.tests import setup
from bibliograph.parsing.tests.base import TestEntries
from bibliograph.parsing.parsers.medline import MedlineParser
from bibliograph.parsing.interfaces import IBibliographyParser

class TestMedlineParser(unittest.TestCase):

    def setUp(self):
        self.parser = MedlineParser()

    def test_parser_contract(self):
        self.failUnless(IBibliographyParser.providedBy(self.parser))
        self.failUnless(verifyObject(IBibliographyParser, self.parser))

    def test_import(self):
        source = open(setup.MEDLINE_TEST_MED, 'r').read()
        self.failUnless(source)

        parser = MedlineParser()
        entries = TestEntries(parser.getEntries(source))
        self.failUnless(len(entries) == 4)
        expected_author_last_names = ("Groot","Bruins","Breeuwer","Alibardi",\
                                          "Thompson","Coke","Hunter","Isaza",\
                                          "Koch","Goatley","Carpenter",\
                                          "Trape","Mane",)

        parsed_author_last_names = entries.author_last_names()
        for name in expected_author_last_names:
            self.failUnless(name in parsed_author_last_names,
                            'Parse failed - missing author %s' % name)
        
    def test_check_format(self):
        s0 = open(setup.MEDLINE_TEST_MED, 'r').read()
        s1 = open(setup.RIS_SOURCE, 'r').read()
        s3 = open(setup.ENDNOTE_TEST_SOURCE, 'r').read()
        
        self.failUnless(self.parser.checkFormat("AB  -\nAU  -\nPMID-\nTI  -"))
        self.failUnless(self.parser.checkFormat(s0), 
                        'Medline Parser failed to detect Medline format')
        self.failIf(self.parser.checkFormat(s1), 
                        'Medline Parser incorrectly accepted RIS format')
        self.failIf(self.parser.checkFormat(s3), 
                    'Medline Parser incorrectly accepted EndNote format')

def test_suite():
    suite = unittest.TestSuite([
        unittest.makeSuite(TestMedlineParser),])
    return suite



import os
from setuptools import setup, find_packages

version = '0.2.2'

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('bibliograph', 'parsing', 'README.txt')
    + '\n' +
    'Change history\n'
    '==============\n'
    + '\n' + 
    read('CHANGES.txt')
    )

setup(name='bibliograph.parsing',
      version=version,
      description="Parsers for bibliograph packages",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='bibtex bibliography xml endnote ris bibutils parsers',
      author='Paul Bugni, Cris Ewing',
      author_email='plone-biblio@das-netzwerkteam.de',
      url='http://svn.plone.org/svn/collective/bibliograph.parsing/',
      license='ZPL 2.1',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['bibliograph'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'bibliograph.core',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import unittest

from zope.app.testing import ztapi
from zope.app.testing.placelesssetup import setUp, tearDown
from zope.interface.verify import verifyObject

from bibliograph.parsing.tests import setup
from bibliograph.parsing.tests.base import TestEntries
from bibliograph.parsing.interfaces import IBibliographyParser
from bibliograph.parsing.parsers.ris import RISParser

class testRISParser(unittest.TestCase):
    """tests to cover the RIS parser"""

    def setUp(self):
        setUp()
        from bibliograph.rendering.interfaces import IBibTransformUtility
        from bibliograph.rendering.utility import ExternalTransformUtility    
        ztapi.provideUtility(IBibTransformUtility, ExternalTransformUtility(),
                             name=u'external')

    def tearDown(self):
        tearDown()

    def test_parser_contract(self):
        self.failUnless(IBibliographyParser.providedBy(RISParser()))
        self.failUnless(verifyObject(IBibliographyParser, RISParser()))
    
    def test_Parser(self):
        """test the functioning of the parser"""
        parser = RISParser()
        if parser.isAvailable():
            source = open(setup.RIS_SOURCE, 'r').read()
            self.failUnless(source)
            
            entries = TestEntries(parser.getEntries(source))
            self.failUnless( len(entries) == 7 )
            self.failUnless( 'Markets and Municipalities: A Study of the Behavior of the Danish Municipalities' in entries.titles())
            
            entry = entries.entries[0]
            self.failUnless( entry.pages == '79--102' )
            self.failUnless( entry.volume == '114' )
            self.failUnless( entry.number == '1 - 2' )
            self.failUnless( entry.publication_year == '2003' )
            self.failUnless( entry.journal == 'Public Choice' )
            # XXX This test could be adjusted if the parser got smarter about
            #     about converting 'Mar.' into 'March'
            ## rr: which it did inbetween it seems because I had to add
            ## the fully spelled out month names to the 'month_mapper' dict.
            self.failUnless( entry.publication_month == '01' )

            entry_authors = entries.entries[0].authors
            self.failUnless( len( entry_authors ) == 2 )
            self.failUnless( entry_authors[0]['lastname'] == 'Christoffersen' )
            self.failUnless( entry_authors[0]['firstname'] == 'Henrik' )
            self.failUnless( entry_authors[0]['middlename'] == '' )
            self.failUnless( entry_authors[1]['lastname'] == 'Paldam' )
            self.failUnless( entry_authors[1]['firstname'] == 'Martin' )
            self.failUnless( entry_authors[1]['middlename'] == '' )
        else:
            print """\nOne or more transformationtool was not found!
please make sure bibutils is installed to run all tests. """
            print ("-" * 20) + "\n"
            
    def test_FormatDetection(self):
        parser = RISParser()
        
        s1 = open(setup.RIS_SOURCE, 'r').read()
        s2 = open(setup.ENDNOTE_TEST_SOURCE, 'r').read()
        
        self.failUnless(parser.checkFormat(s1), 'RIS Parser failed to detect RIS format')
        self.failIf(parser.checkFormat(s2), 'RIS Parser incorrectly detected EndNote format as RIS')
    

def test_suite():
    suite = unittest.TestSuite([
        unittest.makeSuite(testRISParser),])
    return suite



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""BibtexParser class"""

# Python stuff
import re

# Bibliography stuff
from bibliograph.parsing.parsers.base import BibliographyParser

from bibliograph.core.utils import _encode, _decode
from bibliograph.core.encodings import _latex2utf8enc_mapping
from bibliograph.core.encodings import _latex2utf8enc_mapping_simple


_encoding = 'utf-8'   # XXX: should be taken from the site configuration

class BibtexParser(BibliographyParser):
    """
    A specific parser to process input in BiBTeX-format.
    """

    meta_type = "Bibtex Parser"

    format = {'name':'BibTeX',
              'extension':'bib'}

    def __init__(self,
                 id = 'bibtex',
                 title = "BibTeX parser",
                 delimiter = '}\s*@',
                 pattern = '(,\s*[\w\-]{2,}\s*=)'):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        self.setPattern(pattern)


    # Here we need to provide 'checkFormat' and 'parseEntry'
    def checkFormat(self, source):
        """
        is this my format?
        """
        pattern = re.compile('^@[A-Z|a-z]*{', re.M)
        all_tags = re.findall(pattern, source)

        if all_tags:
            for t in all_tags:
                type = t.strip('@{').lower()
                if type not in ('article','book','booklet','conference','inbook','incollection',
                                'inproceedings','manual','mastersthesis','misc','phdthesis',
                                'proceedings','techreport','unpublished','collection','patent',
                                'webpublished'):
                    return 0
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        expands LaTeX macros
        removes LaTeX commands and special formating
        converts special characters to their HTML equivalents
        """
        source = self.expandMacros(source)
        source = self.stripComments(source)
        source = self.convertChars(source)
        # it is important to convertChars before stripping off commands!!!
        # thus, whatever command will be replaced by a unicode value... the
        # remaining LaTeX commands will vanish here...
        source = self.stripCommands(source)
        return source

    def expandMacros(self, source):
        source = self.expandStringMacros(source)
        # add more macro conventions here if there are any
        return source

    def expandStringMacros(self, source):
        lines = source.split('\n')
        macros = []
        sourcelns = []
        for line in lines:
            if line.find('@String') > -1:
                macros.append(line)
            else:
                sourcelns.append(line)
        source = '\n'.join(sourcelns)
        for macro in macros:
            split_on = re.compile('[{=}]+')
            raw_matches = split_on.split(macro)
            matches = [m for m in raw_matches if m not in ['', ' ', '\r']]
            # raise str(matches)
            short = matches[1].strip()
            long = matches[-1].strip()
            pattern = "\\b" + short + "\\b"
            old = re.compile(pattern)
            source = old.sub(long, source)
        return source

    def stripCommands(self, source):
        oldstyle_cmd = re.compile(r'{\\[a-zA-Z]{2,}')
        newstyle_cmd = re.compile(r'\\[a-zA-Z]+{')
        source = oldstyle_cmd.sub('{', source)
        source = newstyle_cmd.sub('{', source)
        return source

    def stripComments(self, source):

        inside_entry = False
        waiting_for_first_brace = False
        newsource = ''

        for idx in range(len(source)):

            char = source[idx]
            last_char = (idx > 0) and source[idx-1] or '\n'
            next_char = (idx < len(source)-1) and source[idx+1] or '\n'

            if char == '@' and not inside_entry:
                inside_entry = True
                waiting_for_first_brace = True
                braces_nesting_level = 0

            if inside_entry:

                newsource = newsource + char
                if char == '{' and last_char != "\\":
                    braces_nesting_level += 1

                if char == '}' and last_char != "\\":
                    braces_nesting_level -= 1

                if waiting_for_first_brace and (braces_nesting_level == 1):
                    waiting_for_first_brace = False

                if (braces_nesting_level == 0) and not waiting_for_first_brace and (char == '}'):
                    inside_entry = False
                    newsource = newsource + "\n"

        completely = re.compile('.*')
        ## this line caused issue #20, leaving it here for now... mg-20061023
        # source = completely.sub(newsource, 'dummy')
        ## this line fixes issue #20
        source = newsource
        return source

    def convertChars(self, source):
        source = self.convertLaTeX2Unicode(source)
        source = self.fixWhiteSpace(source)
        return self.explicitReplacements(source)

    def convertLaTeX2Unicode(self, source):

        for latex_entity in _latex2utf8enc_mapping_simple.keys():
            source = _encode(_decode(source).replace(latex_entity, _latex2utf8enc_mapping_simple[latex_entity]))

        for latex_entity in _latex2utf8enc_mapping.keys():
            source = _encode(_decode(source).replace(latex_entity, _latex2utf8enc_mapping[latex_entity]))

        return source

    def fixWhiteSpace(self, source):
        ttable = [(r'\ ', ' '),
                  (r'\!', ' '),
                  ]
        source = self.mreplace(source, ttable)
        wsp_tilde = re.compile(r'[^/\\]~')
        return wsp_tilde.sub(self.tilde2wsp, source).replace('\~', '~')

    def tilde2wsp(self, hit): return hit.group(0)[0] + ' '

    def explicitReplacements(self, source):
        # list of 2 tuples; second element replaces first
        ttable = [(r'\/', ''),
                  (r'\&', '&'),
                  (r'\~', '~'),
                  (r'---', '&mdash;'),
                  (r'--', '&ndash;'),
                  ]
        return self.mreplace(source, ttable)

    def mreplace(self, s, ttable):
        for a, b in ttable:
            s = s.replace(a, b)
        return s

    # done with preprocessing

    def parseEntry(self, entry):
        """
        parses a single entry

        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        result = {}
        authorlist = []
        authorURLlist = []

        # remove newlines and <CR>s, and remove the last '}'
        entry = entry.replace('\n', ' ').replace('\r', '').replace('\t', ' ').rstrip().rstrip('}')
        tokens = self.pattern.split(entry)
        try:
            type, pid = tokens[0].strip().split('{')
            type = type.replace('@', '').strip().lower()
            result['reference_type'] = type.capitalize() + 'Reference'
            result['pid'] = pid.replace(',', '').strip()
        except:
            return "Bibtex Parser Error: malformed first line."

        for k,v in self.group(tokens[1:],2):
            key = k[1:-1].strip().lower()
            # INBOOKs mapping: title -> booktitle, chapter -> chapter and title
            if type == 'inbook':
                if key == 'title':
                    key = 'booktitle'

                if key == 'chapter':
                    result['title'] = self.cleanLine(v)

            # BibTex field "type" maps to CMFBAT field "publication_type"
            if key == 'type':
                key = 'publication_type'
                result[key] = self.cleanLine(v)

            # special procedure for authors and editors
            elif key == 'author':
                if result.has_key('author'):
                    result[key].append(self.cleanLine(v))
                else:
                    result[key] = [ self.cleanLine(v) ]
            elif (key == 'editor') and (type in ['book','proceedings']):
                if result.has_key('editor'):
                    result[key].append(self.cleanLine(v)) 
                else:
                    result[key] = [self.cleanLine(v)]
            elif key == 'keywords':
                if not result.has_key(key):
                    # Original BibTeX files contain only *one* 'keywords = '
                    # for multiple keywords
                    result[key] = self.splitMultiple(v) 
                else:
                    # This is likely used by other importers/parser trying to mis-use
                    # the BibTeX importer with multiple keywords
                    result[key].append(self.cleanLine(v))
            else:
                value = self.cleanLine(v)
                result[key] = value
                # Aliasing the value to an upper-cased key so that when this dictionary
                # is passed into <a_reference_object>.edit(**dictionary), the values
                # will match and auto-update schema fields that are specified in either
                # upper or lower case.  This is motivated by the 'DOI' field being upper-cased.
                # Of course, this won't help mixed-case fields, but we'd probably need to change
                # Archetype internals to fix that - and that would be a questionable endeavour.
                result[key.upper()] = value

            #print key, result[key]

        # compile authors list of dictionaries
        # we can have authors
        if result.has_key('author'):
            for each in result['author']:
                each = each.replace(' AND', ' and')
                authorlist.extend( each.split(' and') )
        # but for some bibref types we can have editors alternatively
        elif result.has_key('editor') and (type in ['book','proceedings']):
            result['editor_flag'] = True
            for each in result['editor']:
                each = each.replace(' AND', ' and')
                authorlist.extend( each.split(' and') )
        if result.has_key('authorURLs'):
            authorURLlist = result['authorURLs'].split('and ')

        if authorlist:
            alist = []
            authorlist = [x for x in authorlist if x]
            for author in authorlist:
                fname = mname = lname = ''
                parts = self.splitAuthor(author)
                if len(parts) == 1:
                    lname = parts[0].strip()
                else:
                    lname = parts[-1].strip()
                    fname = parts[0].strip()
                    if parts[1:-1]:
                        for part in parts[1:-1]:
                            mname = mname + part.strip()
                adict = {'firstname': fname,
                         'middlename': mname,
                         'lastname': lname}
                alist.append(adict)

        if authorURLlist and alist:
            index = 0
            for url in authorURLlist:
                alist[index]['homepage'] = url.strip()
                index += 1

        if authorlist:
            result['authors'] = alist

        # do some renaming and reformatting
        tmp = result.get('note')
        while tmp and tmp[-1] in ['}', ',', '\n', '\r']:
            tmp = tmp[:-1]
        if tmp:
            result['note'] = tmp
        result['publication_year'] = result.get('year', '')
        result['publication_month'] = result.get('month', '')
        result['publication_url'] = result.get('url', '')
        ## result['publication_title'] = result.get('title', '')
        tmp = result.get('title','')
        for car in ('\n', '\r', '\t'):
            tmp = tmp.replace(car, ' ')
        while '  ' in tmp:
            tmp = tmp.replace('  ', ' ')
        result['title'] = tmp

        # collect identifiers
        identifiers = list()
        for key in ('isbn', 'doi', 'asin', 'purl', 'urn', 'issn'):
            if key in result:
                identifiers.append({'label' : key.upper(), 'value': result[key]})
        if identifiers:
            result['identifiers'] = identifiers

        return result

    # the helper method's

    def splitAuthor(self, author=None):
        if not author: return []
        #parts = author.replace('.', ' ').split(',',1)
        parts = author.split(',',1)
        if len(parts) == 1: return parts[0].split()
        else:
            tmp = parts[1].split()
            tmp.append(parts[0])
            return tmp

    def splitMultiple(self, value):
        value = self.clean(value)
        result = list()
        for item in value.split(','):
            item = item.strip()
            if item:
                result.append(item)
        return result

    def clean(self, value):
        value = value.replace('{', '').replace('}', '').strip()
        if value and value[0] == '"' and len(value) > 1:
            value = value[1:-1]
        return value

    def cleanLine(self, value):
        return self.clean(value.rstrip().rstrip(',').rstrip())

    def group(self, p,n):
        """ Group a sequence p into a list of n tuples."""
        mlen, lft = divmod(len(p), n)
        if lft != 0: mlen += 1

        # initialize a list of suitable length
        lst = [[None]*n for i in range(mlen)]

        # Loop over all items in the input sequence
        for i in range(len(p)):
            j,k = divmod(i,n)
            lst[j][k] = p[i]

        return map(tuple, lst)



import unittest

from zope.app.testing import ztapi
from zope.app.testing.placelesssetup import setUp, tearDown
from zope.interface.verify import verifyObject

from bibliograph.parsing.tests import setup
from bibliograph.parsing.tests.base import TestEntries
from bibliograph.parsing.interfaces import IBibliographyParser
from bibliograph.parsing.parsers.xml import XMLParser

class testXMLParser(unittest.TestCase):
    """tests to cover the XML parser"""

    def setUp(self):
        setUp()
        from bibliograph.rendering.interfaces import IBibTransformUtility
        from bibliograph.rendering.utility import ExternalTransformUtility    
        ztapi.provideUtility(IBibTransformUtility, ExternalTransformUtility(),
                             name=u'external')

    def tearDown(self):
        tearDown()

    def test_parser_contract(self):
        self.failUnless(IBibliographyParser.providedBy(XMLParser()))
        self.failUnless(verifyObject(IBibliographyParser, XMLParser()))
    
    def test_Parser(self):
        """test the functioning of the parser"""
        parser = XMLParser()
        if parser.isAvailable():
            source = open(setup.MEDLINE_TEST_XML, 'r').read()
            self.failUnless(source)
            
            entries = TestEntries(parser.getEntries(source))
            
            # are there enough entries?
            self.failUnless(len(entries) == 4)
            # are there the right number of authors?
            self.failUnless(len(entries.author_last_names()) == 13)
            # test individual name parsings:
            #  lastname, one initial
            this_entry = entries.entryByTitle("Molecular genetic evidence for parthenogenesis in the Burmese python, Python molurus bivittatus.")
            expected_author = {"firstname": "E.", "middlename": "","lastname": "Bruins"}
            self.failUnless(this_entry.authorIsPresent(expected_author), "author: %s is not listed" % expected_author)
            # lastname, more than one initial
            expected_author = {"firstname": "T.", "middlename": "V.M.","lastname": "Groot"}
            self.failUnless(this_entry.authorIsPresent(expected_author), "author: %s is not listed" % expected_author)
            # lastname, firstname, middle initial
            this_entry = entries.entryByTitle("Pharmacokinetics and tissue concentrations of azithromycin in ball pythons (Python regius).")
            expected_author = {"firstname": "Robert", "middlename": "P.","lastname": "Hunter"}
            self.failUnless(this_entry.authorIsPresent(expected_author), "author: %s is not listed" % expected_author)
            
                                
        else:
            print """\nOne or more transformationtool was not found!
please make sure bibutils is installed to run all tests. """
            print ("-" * 20) + "\n"
            
    def test_FormatDetection(self):
        parser = XMLParser()
    
        s1 = open(setup.MEDLINE_TEST_XML, 'r').read()
        s2 = open(setup.MEDLINE_TEST_BIB, 'r').read()
        s3 = open(setup.MEDLINE_TEST_MED, 'r').read()
        
        self.failUnless(parser.checkFormat(s1), 'XML Parser failed to detect XML(MODS) format')
        self.failIf(parser.checkFormat(s2), 'XML Parser incorrectly detected Bibtex format as XML(MODS)')
        self.failIf(parser.checkFormat(s3), 'XML Parser incorrectly detected Medline format as XML(MODS)')
    

def test_suite():
    suite = unittest.TestSuite([
        unittest.makeSuite(testXMLParser),])
    return suite